@@ -18,114 +18,100 @@
  */
 package org.apache.commons.compress;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.fail;
+
 import java.io.BufferedInputStream;
+import java.io.Closeable;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.net.URI;
+import java.net.URL;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.ArrayList;
-import java.util.Iterator;
+import java.util.Arrays;
 import java.util.List;
-import java.util.Locale;
-
-import junit.framework.TestCase;
 
 import org.apache.commons.compress.archivers.ArchiveEntry;
 import org.apache.commons.compress.archivers.ArchiveInputStream;
 import org.apache.commons.compress.archivers.ArchiveOutputStream;
 import org.apache.commons.compress.archivers.ArchiveStreamFactory;
-import org.apache.commons.compress.utils.IOUtils;
+import org.apache.commons.lang3.SystemUtils;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
 
-public abstract class AbstractTestCase extends TestCase {
+public abstract class AbstractTestCase {
 
-    protected File dir;
-    protected File resultDir;
-
-    private File archive; // used to delete the archive in tearDown
-    protected List<String> archiveList; // Lists the content of the archive as originally created
-    
-    protected ArchiveStreamFactory factory = new ArchiveStreamFactory();
-
-    public AbstractTestCase() {
-        super();
+    protected interface StreamWrapper<I extends InputStream> {
+        I wrap(InputStream in) throws Exception;
     }
 
-    public AbstractTestCase(String name) {
-        super(name);
-    }
-
-    @Override
-    protected void setUp() throws Exception {
-        dir = mkdir("dir");
-        resultDir = mkdir("dir-result");
-        archive = null;
-    }
-
-    protected static File mkdir(String name) throws IOException {
-        File f = File.createTempFile(name, "");
-        f.delete();
-        f.mkdir();
-        return f;
-    }
-
-    protected File getFile(String path) {
-        return new File("src/test/resources", path);
-    }
-
-    @Override
-    protected void tearDown() throws Exception {
-        rmdir(dir);
-        rmdir(resultDir);
-        dir = resultDir = null;
-        if (!tryHardToDelete(archive)) {
-            // Note: this exception won't be shown if the test has already failed
-            throw new Exception("Could not delete "+archive.getPath());
+    public static File getFile(final String path) throws IOException {
+        final URL url = AbstractTestCase.class.getClassLoader().getResource(path);
+        if (url == null) {
+            throw new FileNotFoundException("couldn't find " + path);
+        }
+        try {
+            URI uri = url.toURI();
+            return new File(uri);
+        } catch (final java.net.URISyntaxException ex) {
+            throw new IOException(ex);
         }
     }
 
-    protected static void rmdir(File f) {
-        String[] s = f.list();
+    public static Path getPath(final String path) throws IOException {
+        return getFile(path).toPath();
+    }
+
+    public static File mkdir(final String name) throws IOException {
+        return Files.createTempDirectory(name).toFile();
+    }
+
+    public static InputStream newInputStream(final String path) throws IOException {
+        return Files.newInputStream(getPath(path));
+    }
+
+    public static void rmdir(final File f) {
+        final String[] s = f.list();
         if (s != null) {
-            for (int i = 0; i < s.length; i++) {
-                final File file = new File(f, s[i]);
-                if (file.isDirectory()){
+            for (final String element : s) {
+                final File file = new File(f, element);
+                if (file.isDirectory()) {
                     rmdir(file);
                 }
-                boolean ok = tryHardToDelete(file);
-                if (!ok && file.exists()){
-                    System.out.println("Failed to delete "+s[i]+" in "+f.getPath());
+                final boolean ok = tryHardToDelete(file);
+                if (!ok && file.exists()) {
+                    System.out.println("Failed to delete " + element + " in " + f.getPath());
                 }
             }
         }
         tryHardToDelete(f); // safer to delete and check
-        if (f.exists()){
-            throw new Error("Failed to delete "+f.getPath());
+        if (f.exists()) {
+            fail("Failed to delete " + f.getPath());
         }
     }
 
-    private static final boolean ON_WINDOWS =
-        System.getProperty("os.name").toLowerCase(Locale.ENGLISH)
-        .indexOf("windows") > -1;
-
     /**
      * Accommodate Windows bug encountered in both Sun and IBM JDKs.
      * Others possible. If the delete does not work, call System.gc(),
      * wait a little and try again.
      *
+     * Copied from FileUtils in Ant 1.8.0.
+     *
      * @return whether deletion was successful
-     * @since Stolen from FileUtils in Ant 1.8.0
      */
-    public static boolean tryHardToDelete(File f) {
+    public static boolean tryHardToDelete(final File f) {
         if (f != null && f.exists() && !f.delete()) {
-            if (ON_WINDOWS) {
+            if (SystemUtils.IS_OS_WINDOWS) {
                 System.gc();
             }
             try {
                 Thread.sleep(10);
-            } catch (InterruptedException ex) {
+            } catch (final InterruptedException ex) {
                 // Ignore Exception
             }
             return f.delete();
@@ -134,10 +120,142 @@ public abstract class AbstractTestCase extends TestCase {
     }
 
     /**
-     * Creates an archive of textbased files in several directories. The
-     * archivername is the factory identifier for the archiver, for example zip,
+     * Accommodate Windows bug encountered in both Sun and IBM JDKs.
+     * Others possible. If the delete does not work, call System.gc(),
+     * wait a little and try again.
+     *
+     * Copied from FileUtils in Ant 1.8.0.
+     *
+     * @return whether deletion was successful
+     */
+    public static boolean tryHardToDelete(final Path f) {
+        return tryHardToDelete(f != null ? f.toFile() : null);
+    }
+
+    protected File dir;
+
+    protected File resultDir;
+
+    private Path archive; // used to delete the archive in tearDown
+
+    protected List<String> archiveList; // Lists the content of the archive as originally created
+
+    protected final ArchiveStreamFactory factory = ArchiveStreamFactory.DEFAULT;
+
+    /**
+     * Add an entry to the archive, and keep track of the names in archiveList.
+     *
+     * @param out
+     * @param filename
+     * @param infile
+     * @throws IOException
+     * @throws FileNotFoundException
+     */
+    private void addArchiveEntry(final ArchiveOutputStream out, final String filename, final File infile)
+            throws IOException, FileNotFoundException {
+        final ArchiveEntry entry = out.createArchiveEntry(infile, filename);
+        out.putArchiveEntry(entry);
+        Files.copy(infile.toPath(), out);
+        out.closeArchiveEntry();
+        archiveList.add(filename);
+    }
+
+    /**
+     * Checks that an archive input stream can be read, and that the file data matches file sizes.
+     *
+     * @param in
+     * @param expected list of expected entries or {@code null} if no check of names desired
+     * @throws Exception
+     */
+    protected void checkArchiveContent(final ArchiveInputStream in, final List<String> expected)
+            throws Exception {
+        checkArchiveContent(in, expected, true);
+    }
+
+    /**
+     * Checks that an archive input stream can be read, and that the file data matches file sizes.
+     *
+     * @param in
+     * @param expected list of expected entries or {@code null} if no check of names desired
+     * @param cleanUp Cleans up resources if true
+     * @return returns the created result file if cleanUp = false, or null otherwise
+     * @throws Exception
+     */
+    protected File checkArchiveContent(final ArchiveInputStream in, final List<String> expected, final boolean cleanUp)
+            throws Exception {
+        final File result = mkdir("dir-result");
+        result.deleteOnExit();
+
+        try {
+            ArchiveEntry entry;
+            while ((entry = in.getNextEntry()) != null) {
+                final File outfile = new File(result.getCanonicalPath() + "/result/" + entry.getName());
+                long copied = 0;
+                if (entry.isDirectory()) {
+                    outfile.mkdirs();
+                } else {
+                    outfile.getParentFile().mkdirs();
+                    copied = Files.copy(in, outfile.toPath());
+                }
+                final long size = entry.getSize();
+                if (size != ArchiveEntry.SIZE_UNKNOWN) {
+                    assertEquals(size, copied, "Entry.size should equal bytes read.");
+                }
+
+                if (!outfile.exists()) {
+                    fail("extraction failed: " + entry.getName());
+                }
+                if (expected != null && !expected.remove(getExpectedString(entry))) {
+                    fail("unexpected entry: " + getExpectedString(entry));
+                }
+            }
+            in.close();
+            if (expected != null && !expected.isEmpty()) {
+                fail(expected.size() + " missing entries: " + Arrays.toString(expected.toArray()));
+            }
+            if (expected != null) {
+                assertEquals(0, expected.size());
+            }
+        } finally {
+            if (cleanUp) {
+                rmdir(result);
+            }
+        }
+        return result;
+    }
+
+    /**
+     * Checks if an archive contains all expected files.
+     *
+     * @param archive
+     *            the archive to check
+     * @param expected
+     *            a list with expected string file names
+     * @throws Exception
+     */
+    protected void checkArchiveContent(final File archive, final List<String> expected)
+            throws Exception {
+        try (InputStream is = Files.newInputStream(archive.toPath());
+            final ArchiveInputStream in = factory.createArchiveInputStream(new BufferedInputStream(is))) {
+            this.checkArchiveContent(in, expected);
+        }
+    }
+
+    protected void closeQuietly(final Closeable closeable){
+        if (closeable != null) {
+            try {
+                closeable.close();
+            } catch (final IOException ignored) {
+                // ignored
+            }
+        }
+    }
+
+    /**
+     * Creates an archive of text-based files in several directories. The
+     * archivename is the factory identifier for the archiver, for example zip,
      * tar, cpio, jar, ar. The archive is created as a temp file.
-     * 
+     *
      * The archive contains the following files:
      * <ul>
      * <li>testdata/test1.xml</li>
@@ -150,22 +268,22 @@ public abstract class AbstractTestCase extends TestCase {
      * <li>something/bla</li>
      * <li>test with spaces.txt</li>
      * </ul>
-     * 
+     *
      * @param archivename
      *            the identifier of this archive
      * @return the newly created file
      * @throws Exception
      *             in case something goes wrong
      */
-    protected File createArchive(String archivename) throws Exception {
+    protected Path createArchive(final String archivename) throws Exception {
         ArchiveOutputStream out = null;
         OutputStream stream = null;
         try {
-            archive = File.createTempFile("test", "." + archivename);
-            archive.deleteOnExit();
-            archiveList = new ArrayList<String>();
+            archive = Files.createTempFile("test", "." + archivename);
+            archive.toFile().deleteOnExit();
+            archiveList = new ArrayList<>();
 
-            stream = new FileOutputStream(archive);
+            stream = Files.newOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
 
             final File file1 = getFile("test1.xml");
@@ -196,37 +314,20 @@ public abstract class AbstractTestCase extends TestCase {
         }
     }
 
-    /**
-     * Add an entry to the archive, and keep track of the names in archiveList.
-     * 
-     * @param out
-     * @param file1
-     * @throws IOException
-     * @throws FileNotFoundException
-     */
-    private void addArchiveEntry(ArchiveOutputStream out, String filename, final File infile)
-            throws IOException, FileNotFoundException {
-        ArchiveEntry entry = out.createArchiveEntry(infile, filename);
-        out.putArchiveEntry(entry);
-        IOUtils.copy(new FileInputStream(infile), out);
-        out.closeArchiveEntry();
-        archiveList.add(filename);
-    }
-
     /**
      * Create an empty archive.
      * @param archivename
      * @return the archive File
      * @throws Exception
      */
-    protected File createEmptyArchive(String archivename) throws Exception {
+    protected Path createEmptyArchive(final String archivename) throws Exception {
         ArchiveOutputStream out = null;
         OutputStream stream = null;
-        archiveList = new ArrayList<String>();
+        archiveList = new ArrayList<>();
         try {
-            archive = File.createTempFile("empty", "." + archivename);
-            archive.deleteOnExit();
-            stream = new FileOutputStream(archive);
+            archive = Files.createTempFile("empty", "." + archivename);
+            archive.toFile().deleteOnExit();
+            stream = Files.newOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
             out.finish();
         } finally {
@@ -241,19 +342,19 @@ public abstract class AbstractTestCase extends TestCase {
 
     /**
      * Create an archive with a single file "test1.xml".
-     * 
+     *
      * @param archivename
      * @return the archive File
      * @throws Exception
      */
-    protected File createSingleEntryArchive(String archivename) throws Exception {
+    protected Path createSingleEntryArchive(final String archivename) throws Exception {
         ArchiveOutputStream out = null;
         OutputStream stream = null;
-        archiveList = new ArrayList<String>();
+        archiveList = new ArrayList<>();
         try {
-            archive = File.createTempFile("empty", "." + archivename);
-            archive.deleteOnExit();
-            stream = new FileOutputStream(archive);
+            archive = Files.createTempFile("empty", "." + archivename);
+            archive.toFile().deleteOnExit();
+            stream = Files.newOutputStream(archive);
             out = factory.createArchiveOutputStream(archivename, stream);
             // Use short file name so does not cause problems for ar
             addArchiveEntry(out, "test1.xml", getFile("test1.xml"));
@@ -268,109 +369,10 @@ public abstract class AbstractTestCase extends TestCase {
         return archive;
     }
 
-    /**
-     * Checks if an archive contains all expected files.
-     * 
-     * @param archive
-     *            the archive to check
-     * @param expected
-     *            a list with expected string filenames
-     * @throws Exception
-     */
-    protected void checkArchiveContent(File archive, List<String> expected)
-            throws Exception {
-        final InputStream is = new FileInputStream(archive);
-        try {
-            final BufferedInputStream buf = new BufferedInputStream(is);
-            final ArchiveInputStream in = factory.createArchiveInputStream(buf);
-            this.checkArchiveContent(in, expected);
-        } finally {
-            is.close();
-        }
-    }
-
-    /**
-     * Checks that an archive input stream can be read, and that the file data matches file sizes.
-     * 
-     * @param in
-     * @param expected list of expected entries or <code>null</code> if no check of names desired
-     * @throws Exception
-     */
-    protected void checkArchiveContent(ArchiveInputStream in, List<String> expected)
-            throws Exception {
-        checkArchiveContent(in, expected, true);
-    }
-    
-    /**
-     * Checks that an archive input stream can be read, and that the file data matches file sizes.
-     * 
-     * @param in
-     * @param expected list of expected entries or <code>null</code> if no check of names desired
-     * @param cleanUp Cleans up resources if true 
-     * @return returns the created result file if cleanUp = false, or null otherwise 
-     * @throws Exception
-     */
-    protected File checkArchiveContent(ArchiveInputStream in, List<String> expected, boolean cleanUp)
-            throws Exception {
-        File result = mkdir("dir-result");
-        result.deleteOnExit();
-
-        try {
-            ArchiveEntry entry = null;
-            while ((entry = in.getNextEntry()) != null) {
-                File outfile = new File(result.getCanonicalPath() + "/result/"
-                        + entry.getName());
-                long copied=0;
-                if (entry.isDirectory()){
-                    outfile.mkdirs();
-                } else {
-                    outfile.getParentFile().mkdirs();
-                    OutputStream out = new FileOutputStream(outfile);
-                    try {
-                        copied=IOUtils.copy(in, out);
-                    } finally {
-                        out.close();
-                    }                    
-                }
-                final long size = entry.getSize();
-                if (size != ArchiveEntry.SIZE_UNKNOWN) {
-                    assertEquals("Entry.size should equal bytes read.",size, copied);
-                }
-
-                if (!outfile.exists()) {
-                    fail("extraction failed: " + entry.getName());
-                }
-                if (expected != null && !expected.remove(getExpectedString(entry))) {
-                    fail("unexpected entry: " + getExpectedString(entry));
-                }
-            }
-            in.close();
-            if (expected != null && expected.size() > 0) {
-                for (Iterator<String> iterator = expected.iterator(); iterator.hasNext();) {
-                    String name = iterator.next();
-                    fail("Expected entry: " + name);
-                }
-            }
-            if (expected != null) {
-                assertEquals(0, expected.size());
-            }
-        } finally {
-            if (cleanUp) {
-                rmdir(result);
-            }
-        }
-        return result;
-    }
-
-    /**
-     * Override this method to change what is to be compared in the List.
-     * For example, size + name instead of just name.
-     * 
-     * @param entry
-     * @return returns the entry name
-     */
-    protected String getExpectedString(ArchiveEntry entry) {
-        return entry.getName();
+    protected File createTempDir() throws IOException {
+        final File tmpDir = mkdir("testdir");
+        tmpDir.deleteOnExit();
+        return tmpDir;
     }
 
     /**
@@ -379,16 +381,41 @@ public abstract class AbstractTestCase extends TestCase {
      * element of the two element array).
      */
     protected File[] createTempDirAndFile() throws IOException {
-        File tmpDir = mkdir("testdir");
-        tmpDir.deleteOnExit();
-        File tmpFile = File.createTempFile("testfile", "", tmpDir);
+        final File tmpDir = createTempDir();
+        final File tmpFile = File.createTempFile("testfile", "", tmpDir);
         tmpFile.deleteOnExit();
-        FileOutputStream fos = new FileOutputStream(tmpFile);
-        try {
-            fos.write(new byte[] {'f', 'o', 'o'});
-            return new File[] {tmpDir, tmpFile};
-        } finally {
-            fos.close();
-        }            
+        try (OutputStream fos = Files.newOutputStream(tmpFile.toPath())) {
+            fos.write(new byte[] { 'f', 'o', 'o' });
+            return new File[] { tmpDir, tmpFile };
+        }
+    }
+
+    /**
+     * Override this method to change what is to be compared in the List.
+     * For example, size + name instead of just name.
+     *
+     * @param entry
+     * @return returns the entry name
+     */
+    protected String getExpectedString(final ArchiveEntry entry) {
+        return entry.getName();
+    }
+
+    @BeforeEach
+    public void setUp() throws Exception {
+        dir = mkdir("dir");
+        resultDir = mkdir("dir-result");
+        archive = null;
+    }
+
+    @AfterEach
+    public void tearDown() throws Exception {
+        rmdir(dir);
+        rmdir(resultDir);
+        dir = resultDir = null;
+        if (!tryHardToDelete(archive)) {
+            // Note: this exception won't be shown if the test has already failed
+            throw new Exception("Could not delete " + archive);
+        }
     }
 }
